<?php

/**
 * Creates a WordPress user account from an LDAP response specified by
 * $ldapUserData.  Assumes that a user account $newUserName does not already
 * exist.
 * 
 * Code courtesy of dwang99 via post at 
 * <code>http://patcavit.com/2005/05/11/wordpress-ldap-and-playing-nicely/</code>
 * 
 * @author - dwang99
 */
function wpmuLdapCreateWPUserFromLdap($newUserName, $newUserPassword, $ldapUserData) {
  global $base, $error, $wpdb, $current_site;

  // we don't actually care about the WP password (since it's LDAP), but we 
  //  need one for WP database
  $sPassword = generate_random_password();
  $user_id = wpmu_create_user( $newUserName, $sPassword, $ldapUserData[LDAP_INDEX_EMAIL] );
  
  if (!$user_id) {
    $error = __('<strong>Error</strong>: Account creation failed.');
    return false;
  }
  
  //Update their first and last name from ldap
  update_usermeta( $user_id, 'first_name', $ldapUserData[LDAP_INDEX_GIVEN_NAME] );
  update_usermeta( $user_id, 'last_name', $ldapUserData[LDAP_INDEX_SURNAME] );
  
  //This is for plugin events
  do_action( 'wpmu_new_user', $user_id );
  do_action('wpmu_activate_user', $user_id, $newUserPassword);
  
  $domain = strtolower( wp_specialchars( $newUserName ) );
  if( constant( "VHOST" ) == 'yes' ) {
    $newdomain = $domain . "." . $current_site->domain;
    $path = $base;
  }
  else {
    $newdomain = $current_site->domain;
    $path = $base . $domain . '/';
  }

  
  // Becuase WPMU has a bug in the create blog function we need this code 
  // To prevent errors with the wpmu_create_blog function.  See Ticket #184
  $result = $wpdb->query( "CREATE TABLE wp_" . $user_id . "_options (option_id bigint(20) NOT NULL auto_increment, blog_id int(11) NOT NULL default 0, option_name varchar(64) NOT NULL default '', option_can_override enum('Y','N') NOT NULL default 'Y', option_type int(11) NOT NULL default 1, option_value longtext NOT NULL, option_width int(11) NOT NULL default 20, option_height int(11) NOT NULL default 8, option_description tinytext NOT NULL, option_admin_level int(11) NOT NULL default 1, autoload enum('yes','no') NOT NULL default 'yes', PRIMARY KEY  (option_id,blog_id,option_name), KEY option_name (option_name)) ENGINE=MyISAM DEFAULT CHARSET=utf8;" );
  
  // Create and update the user's blog.
  $meta = apply_filters('signup_create_blog_meta', array ('lang_id' => 'en', 'public' => 0));
  $blog_id = wpmu_create_blog($newdomain, $path, $newUserName . "'s blog", $user_id, $meta);
  do_action('wpmu_activate_blog', $blog_id, $user_id, $newUserPassword, $newUserName . "'s blog", $meta);
  
  //Must recreated the login object for our shiny NEW users.
  return get_userdatabylogin($newUserName);
}


/**
 * Authenticates an LDAP account using credentials $loginUserName and
 * $loginPassword, returned from an LDAP server accessible from $ldapString.  If
 * a WordPress user account does not exist, and this plugin is configured to
 * create new accounts from LDAP logins, an account will be created.
 */
function wpmuLdapAuthenticate(&$loginObj, $ldapString, $loginUserName, $loginPassword) {
  global $error;
  
  // is it configured to create WP accounts from LDAP accounts?
  $ldapCreateAcct = get_site_option("ldapCreateAcct");

  $server = new LDAP_ro($ldapString);
  $server->DebugOn();
  
  // undefined now - going to populate it in $server->Authenticate
  $userDataArray = null;
  $result = $server->Authenticate ($loginUserName, $loginPassword, $userDataArray);
  
  if ($result == LDAP_OK) {
    // call the registration function to create a wordpress user account for this
    // successfully authenticated user
    require_once( ABSPATH . WPINC . '/registration.php');
    
    // if the account doesn't already exist    
    if ( !username_exists( $loginUserName ) ) {

      // Make the WP users automatically if we're configured to do so      
      if ($ldapCreateAcct ) {
        $loginObj = wpmuLdapCreateWPUserFromLdap($loginUserName, $loginPassword, $userDataArray);
        
        //Setup redirection to user's home directory.
        if (!strpos($_REQUEST['redirect_to'], $loginUserName)) {
          $_REQUEST['redirect_to'] = $loginUserName . "/" . $_REQUEST['redirect_to'];
        }
  
        return true;        
      }
      
      // but if not configured to create 'em, exit with an error
      else {
        $error = __('<strong>Error</strong>: A blogging account does not exist - contact your administrator.');
        return false;
      }
    }
    
    // otherwise, the account *does* exist already, so just get the account info 
    else
      $loginObj = get_userdatabylogin($loginUserName); 
      
    
    // At this point we must have a login object, but just in case something went wrong
    if (!$loginObj) {
      $error = __('<strong>Error</strong>: Unknown error in LDAP Authentication.');
      return false;
    }
    
    // Handle blog removal for various reasons
    $primary_blog = get_usermeta( $login->ID, "primary_blog" );
    if( $primary_blog ) {
      $details = get_blog_details( $primary_blog );
      if( is_object( $details ) ) {
        if( $details->archived == 1 || $details->spam == 1 || $details->deleted == 1 ) {
          $error = __('<strong>Error</strong>: Blog suspended.');
          return false;
        }
      }
    }
    
    // if we get to here - they're authenticated, they have a WP account, so 
    //  it's all set
    return true;
  }
  
  // handle both at once, for security
  else if ( ($result == LDAP_ERROR_USER_NOT_FOUND) || ($result == LDAP_ERROR_WRONG_PASSWORD) ) {
    $error = __('<strong>Error</strong>: Wrong username / password combination.');
    $pwd = '';
    return false;
  }

  // the trickle-through catch-all  
  else {
    $error = __('<strong>Error</strong>: Unknown error in LDAP Authentication.');
    return false;
  }
}


/** 
 * Overrides display and handling of the WPMU signup form.  Simply
 * displays a message to indicate to users that they should use the login form
 * for signup, as the LDAP plugin will automatically create WPMU user accounts
 * and blogs for them.
 * 
 * Based loosely on code from a post by Jeremy Visser at 
 * <code>http://mu.wordpress.org/forums/topic.php?id=2361&replies=15</code>
 * 
 * @author Sean Wedig (www.thecodelife.net)
 */
function wpmuLdapDisableSignup() {
  get_header();
  echo '<div id="content" class="widecolumn">';
  
  $msg = stripslashes(get_site_option('ldapSignupMessage'));
        
  _e($msg);
  echo '</div>';
  get_footer();
  die();
}

?>