<?php
/*
Plugin Name: LDAP Authentication Plug-in
Plugin URI: http://wpmudev.org/project/WPMU-LDAP-Authentication-Plug-in
Description: A plugin to override the core Wordpress MU authentication method (for non-site admins) so as to use an LDAP server for authentication.  
Version: 1.2.0
Author: Alistair Young (http://www.weblogs.uhi.ac.uk/sm00ay/), Patrick Cavit (http://patcavit.com), Hugo Salgado (http://hugo.vulcano.cl), Alex Barker (http://www.callutheran.edu), Dexter Arver, Sean Wedig (http://www.thecodelife.net)
*/

// Includes
require_once("ldap/ldap_ro.php");

// methods for supporting site-admin configuration of the plugin
// TODO: Limit this code to only be called when in the admin interface, instead of with every WPMU call
require_once("ldap/wpmu_ldap_admin.functions.php");
add_action('admin_head', 'ldapShowOptionsCss');
add_action('admin_menu', 'ldap_addmenu');


// *** End Admin Config Functions *** //


// *** Begin User Auth Functions *** //
// disable public signup if configured to do so  
if (get_site_option("ldapDisableSignup") ) {
  add_action('signup_header', 'wpmuLdapDisableSignup');
}

// perform these filters, actions, and WP function overrides only if LDAP-
//  authentication is enabled; this is to cut down on parsing of this code when
//  it doesn't apply 
if (get_site_option("ldapAuth")) { 

  // only include them if it's active, so as to cut down on continual parsing of the code 
  require_once("ldap/wpmu_ldap.functions.php");

  // disable only for site admins
  add_filter('show_password_fields', 'wpmuLdapDisableLdapPassword');
  

  /**
   * Overridden version of WPMU's wp_login method to allow for LDAP
   * authentication for non-site-admin accounts.
   */
  function wp_login($username, $password, $already_md5 = false) {
  	global $wpdb, $error, $current_site, $current_user, $base;
  	
  	// make sure we always use lowercase usernames
  	$username = strtolower($username);
  	
  	$ldapAuth          = get_site_option("ldapAuth");
  	$ldapCookieMarker  = get_site_option("ldapCookieMarker");
  	$ldapServerAddr    = get_site_option("ldapServerAddr");
  	$ldapServerOU      = get_site_option("ldapServerOU");
  	$ldapServerCN      = get_site_option("ldapServerCN");
  	$ldapServerPass    = get_site_option("ldapServerPass");
  	
  	$ldapString 		= "$ldapServerAddr:$ldapServerOU:$ldapServerCN:$ldapServerPass";
  
  	if($ldapAuth && !$ldapCookieMarker) {
  		update_site_option("ldapCookierMarker", "LDAP");
  		$ldapCookieMarker = get_site_option("ldapCookieMarker");
  	}
  	
  	if(!$username) {
  		$error = __('<strong>Error</strong>: The username field is empty.');
  		return false;
  	}
  	
  	if(!$password) {
  		$error = __('<strong>Error</strong>: The password field is empty.');
  		return false;
  	}
  	
  	// if we are already logged in and we try to relogin, just allow it
  	if ($current_user->data->user_login == $username) {
  		return true;
  	}
  	
    // try finding a WP account for this user name
  	$login = get_userdatabylogin($username);
  	
    
  	// Keep the admin user local in case all LDAP servers go down
    // Handle if the user account is not a site admin, or if a WP user does not yet
    //  exist (implicit from "!is_site_admin")
  	if (!is_site_admin($username)) {
  		// If already_md5 is TRUE, then we're getting the user/password from the 
      //  cookie instead of POST. As we don't want to store LDAP passwords in any
  		//  form, we've already replaced the password with the hashed username and 
      //  LDAP_COOKIE_MARKER
  		if ($already_md5 && ($password == md5($username).md5($ldapCookieMarker)) ) {
  				return true;
  		}
  		
  		// No cookie, so authenticate them via LDAP, potentially creating a WP user
      return wpmuLdapAuthenticate($login, $ldapString, $username, $password);      
  	} 
  
    // if it IS a site admin, though, we bypass LDAP entirely - just use WP's auth	
    else {
  
  		// This is all original code pulled from the pluggable.php file located in 
      //  the wp-plugins folder.  Comments were added to help figure out what is 
      //  going on.
  		
  		//Setup some kind of default blog for the user...
  		$primary_blog = get_usermeta( $login->ID, "primary_blog" );
  		if( $primary_blog ) {
  			$details = get_blog_details( $primary_blog );
  			if( is_object( $details ) ) {
  				if( $details->archived == 1 || $details->spam == 1 || $details->deleted == 1 ) {
  					$error = __('<strong>Error</strong>: Blog suspended.');
  					return false;
  				}
  			}
  		}
  		
      
  		//Setup redirection to users home directory.
  		if (!strpos($_REQUEST['redirect_to'], $username)) {
  			$_REQUEST['redirect_to'] = $username . "/" . $_REQUEST['redirect_to'];
  		}
  		
  		// If the password is already_md5, it has been double hashed.
  		// Otherwise, it is plain text.
      
  		if ( ($already_md5 && $login->user_login == $username && md5($login->user_pass) == $password) || 
           ($login->user_login == $username && $login->user_pass == md5($password)) ) {
        return true;
  		} else {
  			$error = __('<strong>Error</strong>: Incorrect password.');
  			$pwd = '';
  			return false;
  		}
  	}
  }


  /**
   * Overridden version of WPMU's wp_setcookie method to allow for different
   * hashing of LDAP-authenticated passwords stored in cookies.  This
   * different hashing will only apply to non-site admins, or to everyone if
   * LDAP authentication is disabled.
   */
  function wp_setcookie($username, $password, $already_md5 = false, $home = '', $siteurl = '', $remember=false) {
  	$ldapCookieMarker = get_site_option("ldapCookieMarker");
  	$ldapAuth = get_site_option("ldapAuth");
  	
    
  	if( ($ldapAuth) && ( !is_site_admin($username) ) ) {
    	// This hanldes all LDAP-authenticated users
  		$password = md5($username).md5($ldapCookieMarker);
  	}
  	else {
  		if(!$already_md5) {
        // Basically this will check admin or users when ldap is disabled.
  			$password = md5( md5($password) ); // Double hash the password in the cookie.
  		}
  	}
  	
  	// From Here Down the File is Stock Form pluggable.php after #125: login-cookie-01.patch
  	if ( $remember )
  		$expire = time() + 31536000;
  	else
  		$expire = 0;
  
  	global $base;
  	setcookie(USER_COOKIE, $username, $expire, $base, COOKIE_DOMAIN);
  	setcookie(PASS_COOKIE, $password, $expire, $base, COOKIE_DOMAIN);
  }
  // *** End User Auth Functions *** //
  
  
}

?>
